import React, { useState, useEffect } from 'react';
import { DrupalJsonApiParams } from 'drupal-jsonapi-params';
import axios from 'axios';
import { Prism as SyntaxHighlighter } from 'react-syntax-highlighter';
import { materialLight } from 'react-syntax-highlighter/dist/esm/styles/prism';

// Import resizable panel components
import {
  Panel,
  PanelGroup,
  PanelResizeHandle
} from 'react-resizable-panels';

// Import custom components
import EntityTypeSelector from './components/EntityTypeSelector';
import FieldsSelector from './components/FieldsSelector';
import IncludeSelector from './components/IncludeSelector';
import FilterSelector from './components/FilterSelector';
import SortSelector from './components/SortSelector';
import PageLimitSelector from './components/PageLimitSelector';
import UrlDisplay from './components/UrlDisplay';
import ResponseDisplay from './components/ResponseDisplay';
import AuthSection from './components/AuthSection';
import RequestHistorySection from './components/RequestHistorySection';
import CodeExamplesTab from './components/CodeExamplesTab';

// Import CSS for resizable panels
import './resizable-panels.css';

// Constants for localStorage keys
const STORAGE_KEYS = {
  REQUEST_HISTORY: 'jsonapi_qb_request_history',
  AUTH_TOKEN: 'jsonapi_qb_auth_token',
  SELECTED_ENTITY: 'jsonapi_qb_selected_entity',
  SELECTED_BUNDLE: 'jsonapi_qb_selected_bundle',
  SELECTED_FIELDS: 'jsonapi_qb_selected_fields',
  INCLUDES: 'jsonapi_qb_includes',
  FILTERS: 'jsonapi_qb_filters',
  SORTS: 'jsonapi_qb_sorts',
  PAGE: 'jsonapi_qb_page',
  LIMIT: 'jsonapi_qb_limit',
  PANEL_LAYOUT: 'jsonapi_qb_panel_layout',
  AUTO_EXECUTE: 'jsonapi_qb_auto_execute'
};

// Helper to safely interact with localStorage
const storage = {
  get: (key, defaultValue = null) => {
    try {
      const item = localStorage.getItem(key);
      return item ? JSON.parse(item) : defaultValue;
    } catch (error) {
      console.error(`Error retrieving ${key} from localStorage:`, error);
      return defaultValue;
    }
  },
  set: (key, value) => {
    try {
      localStorage.setItem(key, JSON.stringify(value));
      return true;
    } catch (error) {
      console.error(`Error storing ${key} in localStorage:`, error);
      return false;
    }
  }
};

function App({ baseUrl = '/jsonapi/', initialActiveTab = 0 }) {
  // Extract the site base URL without jsonapi endpoint for examples
  // Handle both default and custom JSON:API paths
  const jsonapiPathPattern = new RegExp(`${baseUrl.replace(/^\/|\/$/g, '')}/?$`);
  const siteBaseUrl = baseUrl.replace(jsonapiPathPattern, '');
  const [entities, setEntities] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);

  // Panel layout state
  const [activeTab, setActiveTab] = useState(initialActiveTab);
  const [expandedExample, setExpandedExample] = useState('filtering');
  const [builderPanelActiveTab, setBuilderPanelActiveTab] = useState('fields'); // Default to fields tab

  // Default panel sizes with stored values if available
  const savedPanelSizes = storage.get(STORAGE_KEYS.PANEL_LAYOUT, [20, 45, 35]);
  const [panelSizes, setPanelSizes] = useState(savedPanelSizes);

  // Authentication tab state
  const [authTokenTab, setAuthTokenTab] = useState('js');
  const [useTokenTab, setUseTokenTab] = useState('js');

  // Load selections from localStorage with fallbacks to defaults
  const [selectedEntity, setSelectedEntity] = useState(() => storage.get(STORAGE_KEYS.SELECTED_ENTITY, ''));
  const [selectedBundle, setSelectedBundle] = useState(() => storage.get(STORAGE_KEYS.SELECTED_BUNDLE, ''));
  const [selectedFields, setSelectedFields] = useState(() => storage.get(STORAGE_KEYS.SELECTED_FIELDS, []));
  const [includes, setIncludes] = useState(() => storage.get(STORAGE_KEYS.INCLUDES, []));
  const [filters, setFilters] = useState(() => storage.get(STORAGE_KEYS.FILTERS, []));
  const [sorts, setSorts] = useState(() => storage.get(STORAGE_KEYS.SORTS, []));
  const [page, setPage] = useState(() => storage.get(STORAGE_KEYS.PAGE, 0));
  const [limit, setLimit] = useState(() => storage.get(STORAGE_KEYS.LIMIT, 10));

  const [requestUrl, setRequestUrl] = useState('');
  const [response, setResponse] = useState(null);
  const [responseHeaders, setResponseHeaders] = useState(null);
  const [responseSize, setResponseSize] = useState(0);
  const [responseTime, setResponseTime] = useState(0);

  // Load initial auth token from localStorage
  const [authToken, setAuthToken] = useState(() => storage.get(STORAGE_KEYS.AUTH_TOKEN, ''));

  // Load initial request history from localStorage
  const [requestHistory, setRequestHistory] = useState(() => storage.get(STORAGE_KEYS.REQUEST_HISTORY, []));
  const [notification, setNotification] = useState({ open: false, message: '', type: 'info' });
  
  // Auto-execute preference (default to false)
  const [autoExecute, setAutoExecute] = useState(() => storage.get(STORAGE_KEYS.AUTO_EXECUTE, false));

  useEffect(() => {
    // Fetch JSON:API resources on component mount
    fetchEntities();
  }, []);

  // Keep track of fields from related entities
  const [relatedEntityFields, setRelatedEntityFields] = useState({});
  const [loadingRelatedFields, setLoadingRelatedFields] = useState(false);

  // Save panel sizes to localStorage when they change
  const handlePanelResize = (sizes) => {
    setPanelSizes(sizes);
    storage.set(STORAGE_KEYS.PANEL_LAYOUT, sizes);
  };

  // Advanced relationship field loading with better state management
  useEffect(() => {
    // Generate a unique request ID for this run to avoid race conditions
    const requestId = Date.now().toString();
    console.log(`Starting load request #${requestId}`);
    
    const loadIncludeFields = async () => {
      // Only run if we have a valid entity and bundle and includes
      if (!selectedEntity || !selectedBundle || includes.length === 0) {
        if (includes.length === 0) {
          // Clear related fields when no includes are selected
          setRelatedEntityFields({});
        }
        setLoadingRelatedFields(false);
        return;
      }

      // Indicate loading started
      setLoadingRelatedFields(true);
      console.log(`Request #${requestId}: Loading fields for ${includes.length} includes`);

      try {
        // Get current related fields as a base (always start with a clean slate)
        let newRelatedFields = {};

        // Track completed includes for progress
        let completedIncludes = 0;
        const totalIncludes = includes.length;

        // Create an array of promises for parallel fetching
        const fetchPromises = includes.map(async (include) => {
          try {
            console.log(`Request #${requestId}: Processing include: ${include}`);
            // Get the relationship schema URL from the current entity schema
            const schemaUrl = `${baseUrl}${selectedEntity}/${selectedBundle}/resource/schema`;

            // Fetch the schema for current entity
            const response = await fetch(schemaUrl, {
              headers: { 'Accept': 'application/vnd.api+json' }
            });

            if (response.ok) {
              const data = await response.json();

              // Find the relationship definition in the schema
              const includeDef = data.definitions?.relationships?.properties?.[include];

              if (includeDef) {
                // Extract target type from relationship definition
                let targetType = null;
                let targetBundle = null;

                // Check data property - it contains the type enum
                if (includeDef.properties?.data?.properties?.type?.enum?.[0]) {
                  const fullType = includeDef.properties.data.properties.type.enum[0];
                  [targetType, targetBundle] = fullType.split('--');
                } else if (includeDef.properties?.data?.items?.properties?.type?.enum?.[0]) {
                  // For collections (arrays)
                  const fullType = includeDef.properties.data.items.properties.type.enum[0];
                  [targetType, targetBundle] = fullType.split('--');
                }

                if (targetType && targetBundle) {
                  // Fetch the schema for the target entity
                  const targetSchemaUrl = `${baseUrl}${targetType}/${targetBundle}/resource/schema`;
                  const targetResponse = await fetch(targetSchemaUrl, {
                    headers: { 'Accept': 'application/vnd.api+json' }
                  });

                  if (targetResponse.ok) {
                    const targetData = await targetResponse.json();

                    // Extract fields from target schema's attributes
                    const targetFields = [];

                    // Store the target entity type information to use when formatting field parameters
                    const targetEntityType = `${targetType}--${targetBundle}`;

                    // Store info about this relationship for associating with direct fields later
                    const includeInfo = {
                      targetEntityType: targetEntityType,
                      title: includeDef.title || include,
                      description: includeDef.description || `The ${include} include`
                    };

                    // Add the relationship field to ensure it's available
                    targetFields.push({
                      name: include,
                      // Create basic metadata for the base relationship
                      fieldDef: {
                        title: includeDef.title || `${include} (Include)`,
                        description: includeDef.description || `The ${include} include`,
                        type: 'include'
                      },
                      // Store this as a base include field
                      isBaseInclude: true,
                      // Store target entity type
                      targetEntityType: targetEntityType,
                      // Store include info
                      includeInfo: includeInfo
                    });

                    // Process attributes
                    if (targetData.definitions?.attributes?.properties) {
                      // Get field names from the target entity
                      Object.keys(targetData.definitions.attributes.properties).forEach(fieldName => {
                        // Create the prefixed field name (e.g., uid.name)
                        const prefixedFieldName = `${include}.${fieldName}`;
                        targetFields.push({
                          name: prefixedFieldName,
                          // Include metadata from original field
                          fieldDef: targetData.definitions.attributes.properties[fieldName],
                          // Store include target entity type for later use in field params
                          targetEntityType: targetEntityType,
                          // Store the base include name for UI logic
                          baseInclude: include
                        });
                      });
                    }

                    // Return the fields for this relationship
                    if (targetFields.length > 0) {
                      console.log(`Request #${requestId}: Loaded ${targetFields.length} fields for ${include}`);
                      completedIncludes++;
                      console.log(`Request #${requestId}: Progress ${completedIncludes}/${totalIncludes}`);
                      
                      return { include, fields: targetFields };
                    }
                  }
                }
              }
            }
            // If we get here, we couldn't find the include or its fields
            completedIncludes++;
            console.log(`Request #${requestId}: No fields found for ${include}`);
            return { include, fields: [] };
          } catch (error) {
            completedIncludes++;
            console.error(`Request #${requestId}: Error fetching fields for include ${include}:`, error);
            return { include, fields: [] };
          }
        });

        // Wait for all relationships to be processed in parallel
        const results = await Promise.all(fetchPromises);
        
        // Process results and build the new state
        results.forEach(result => {
          if (result && result.include && result.fields.length > 0) {
            newRelatedFields[result.include] = result.fields;
          }
        });

        // Update the state with the new fields
        console.log(`Request #${requestId}: Setting relatedEntityFields with keys: ${Object.keys(newRelatedFields)}`);
        
        // Set state in a way that ensures we don't miss updates
        setRelatedEntityFields(prevFields => {
          // Merge with any existing fields for relationships that weren't in this batch
          const mergedFields = { ...prevFields };
          
          // Add the newly loaded fields
          Object.keys(newRelatedFields).forEach(rel => {
            mergedFields[rel] = newRelatedFields[rel];
          });
          
          return mergedFields;
        });
      } catch (error) {
        console.error(`Request #${requestId}: Error loading relationship fields:`, error);
      } finally {
        // Clear the loading state after a short delay to ensure rendering completes
        setTimeout(() => {
          console.log(`Request #${requestId}: Finished loading relationship fields`);
          setLoadingRelatedFields(false);
        }, 300);
      }
    };

    // Execute the function
    loadIncludeFields();

    // Cleanup function
    return () => {
      console.log(`Cleaning up request #${requestId}`);
    };
  }, [selectedEntity, selectedBundle, includes, baseUrl]);

  // Use a ref to track if includes have been updated by user action
  const userActionRef = React.useRef(false);
  // Use ref to track if this is the initial render/load from saved state
  const initialLoadRef = React.useRef(true);

  // Add a separate effect to handle changes to includes
  // to clean up fields when relationships are removed
  const prevIncludesRef = React.useRef([]);

  useEffect(() => {
    // If entity and bundle are selected and it's not the initial render
    if (selectedEntity && selectedBundle) {
      // Capture current and previous includes for comparison
      const currentIncludes = new Set(includes);
      const prevIncludes = new Set(prevIncludesRef.current);

      // Find relationships that were removed (in prev but not in current)
      const removedRelationships = prevIncludesRef.current.filter(
        rel => !currentIncludes.has(rel)
      );

      // Find relationships that were added (in current but not in prev)
      const addedRelationships = includes.filter(
        rel => !prevIncludes.has(rel)
      );

      // Store current includes for next comparison
      prevIncludesRef.current = [...includes];

      // Handle removed relationships - clean up related fields
      if (removedRelationships.length > 0) {
        // Clean up selected fields
        setSelectedFields(prevFields =>
          prevFields.filter(field =>
            !removedRelationships.some(rel => field.startsWith(`${rel}.`))
          )
        );

        // Clean up filters
        setFilters(prevFilters =>
          prevFilters.filter(filter =>
            !removedRelationships.some(rel => filter.field?.startsWith(`${rel}.`))
          )
        );

        // Clean up sorts
        setSorts(prevSorts =>
          prevSorts.filter(sort =>
            !removedRelationships.some(rel => sort.field?.startsWith(`${rel}.`))
          )
        );
      }

      // For newly added includes, show notification
      if (addedRelationships.length > 0 && userActionRef.current) {
        // Format the notification message
        let message;
        if (addedRelationships.length === 1) {
          message = `Added "${addedRelationships[0]}" include. New fields, filters, and sort criteria are now available.`;
        } else if (addedRelationships.length === 2) {
          message = `Added "${addedRelationships[0]}" and "${addedRelationships[1]}" includes. New fields, filters, and sort criteria are now available.`;
        } else {
          message = `Added ${addedRelationships.length} includes. New fields, filters, and sort criteria are now available.`;
        }

        // Force a clear and reload of include data
        setRelatedEntityFields({}); // Clear current fields
        setLoadingRelatedFields(true); // Set loading state explicitly
        
        // Show the notification after a short delay
        setTimeout(() => {
          showNotification(message, 'status');
        }, 100);
      }

      // After the first update, mark initial load as complete
      if (initialLoadRef.current) {
        initialLoadRef.current = false;
      }

      // Reset the user action flag
      userActionRef.current = false;
    }
  }, [includes, selectedEntity, selectedBundle]);

  // Use refs outside useEffect for tracking previous values
  const prevEntityRef = React.useRef(selectedEntity);
  const prevBundleRef = React.useRef(selectedBundle);

  // Reset filters, fields, etc. when entity type or bundle changes
  useEffect(() => {
    // Check if entity or bundle changed and we had previous values
    if ((prevEntityRef.current || prevBundleRef.current) && // Had previous values
        (prevEntityRef.current !== selectedEntity || prevBundleRef.current !== selectedBundle) && // Change detected
        (selectedEntity && selectedBundle)) { // Have valid new values

      console.log('Entity or bundle changed, resetting parameters');
      setSelectedFields([]);
      setIncludes([]);
      setFilters([]);
      setSorts([]);
      setPage(0);
      setLimit(10);
    }

    // Update refs for next check
    prevEntityRef.current = selectedEntity;
    prevBundleRef.current = selectedBundle;
  }, [selectedEntity, selectedBundle]);

  useEffect(() => {
    // Build the URL when any of the query parameters change
    buildUrl();
  }, [selectedEntity, selectedBundle, selectedFields, includes, filters, sorts, page, limit]);
  
  // Add auto-execute effect with debounce to prevent too many API calls 
  useEffect(() => {
    if (autoExecute && requestUrl) {
      // Use timeout to debounce requests (wait 300ms after changes before executing)
      const timeout = setTimeout(() => {
        executeRequest();
      }, 300);
      
      // Clean up timeout on component unmount or before next run
      return () => clearTimeout(timeout);
    }
  }, [requestUrl, autoExecute]);

  // Persist auth token whenever it changes
  useEffect(() => {
    if (authToken !== undefined) {
      storage.set(STORAGE_KEYS.AUTH_TOKEN, authToken);
    }
  }, [authToken]);

  // Persist entity and bundle selections
  useEffect(() => {
    if (selectedEntity !== undefined) {
      storage.set(STORAGE_KEYS.SELECTED_ENTITY, selectedEntity);
    }
  }, [selectedEntity]);

  useEffect(() => {
    if (selectedBundle !== undefined) {
      storage.set(STORAGE_KEYS.SELECTED_BUNDLE, selectedBundle);
    }
  }, [selectedBundle]);

  // Persist fields, includes, filters and sorts
  useEffect(() => {
    storage.set(STORAGE_KEYS.SELECTED_FIELDS, selectedFields);
  }, [selectedFields]);

  useEffect(() => {
    storage.set(STORAGE_KEYS.INCLUDES, includes);
  }, [includes]);

  useEffect(() => {
    storage.set(STORAGE_KEYS.FILTERS, filters);
  }, [filters]);

  useEffect(() => {
    storage.set(STORAGE_KEYS.SORTS, sorts);
  }, [sorts]);

  // Persist pagination options
  useEffect(() => {
    storage.set(STORAGE_KEYS.PAGE, page);
  }, [page]);

  useEffect(() => {
    storage.set(STORAGE_KEYS.LIMIT, limit);
  }, [limit]);
  
  // Persist autoExecute preference
  useEffect(() => {
    storage.set(STORAGE_KEYS.AUTO_EXECUTE, autoExecute);
  }, [autoExecute]);

  const fetchEntities = async () => {
    setLoading(true);
    setError(null);

    try {
      const response = await axios.get(baseUrl);
      if (response.data && response.data.links) {
        const entityTypes = extractEntityTypes(response.data);
        setEntities(entityTypes);
      } else {
        setError('Invalid JSON:API response format');
      }
    } catch (err) {
      setError(`Error fetching JSON:API resources: ${err.message}`);
    } finally {
      setLoading(false);
    }
  };

  const extractEntityTypes = (data) => {
    const entityTypes = [];

    // Extract entity types from JSON:API response
    if (data && data.links) {
      Object.keys(data.links).forEach(link => {
        if (link !== 'self') {
          const parts = link.split('--');
          if (parts.length === 2) {
            const [entityType, bundle] = parts;

            // Check if entity type already exists
            const existingType = entityTypes.find(type => type.name === entityType);
            if (existingType) {
              existingType.bundles.push({
                name: bundle,
                href: data.links[link].href
              });
            } else {
              entityTypes.push({
                name: entityType,
                bundles: [{
                  name: bundle,
                  href: data.links[link].href
                }]
              });
            }
          }
        }
      });
    }

    return entityTypes;
  };

  const buildUrl = () => {
    if (!selectedEntity || !selectedBundle) {
      setRequestUrl('');
      return;
    }

    // Initialize the JSON:API parameters builder
    const apiParams = new DrupalJsonApiParams();

    // Add entity type for fields
    const entityType = `${selectedEntity}--${selectedBundle}`;

    // Fields - Process them by entity type
    if (selectedFields.length > 0) {
      // Group fields by their entity types
      const fieldsByEntityType = {};

      // Initialize with the main entity type
      fieldsByEntityType[entityType] = [];

      // Process each field
      selectedFields.forEach(field => {
        // Check if it's a relationship field (contains a dot)
        if (field.includes('.')) {
          // Split into relationship name and field name
          const [relationshipName, fieldName] = field.split('.');

          // Find target entity type for this relationship
          if (relationshipName && fieldName && relatedEntityFields[relationshipName]) {
            // Extract the first field's full metadata to get target type information
            const sampleField = relatedEntityFields[relationshipName][0];
            if (sampleField && sampleField.fieldDef) {
              // Extract field definition which should have type information
              console.log('Sample field metadata:', sampleField);

              // Use relationship info to determine the entity type and bundle
              // Try to extract from the loaded relationship schema
              let targetType = Object.keys(apiParams.data.fields).find(type =>
                type.startsWith(relationshipName + '--')
              );

              // If not found in already defined fields, check if we have the info from schema loading
              if (!targetType) {
                // Use a method to find the target entity type based on the relationship name
                // This requires the relatedEntityFields to contain target type information
                const targetEntityInfo = findTargetEntityTypeFromRelationship(relationshipName);
                if (targetEntityInfo) {
                  targetType = targetEntityInfo;
                } else {
                  console.warn(`Could not determine target entity type for relationship: ${relationshipName}`);
                  // Show a notification to the user
                  showNotification(`Could not determine entity type for relationship field "${field}". Field selection will be ignored in the API request.`, 'warning');
                  // Don't add the field to any entity type as it would be formatted incorrectly
                  return;
                }
              }

              // Initialize if this entity type hasn't been seen before
              if (!fieldsByEntityType[targetType]) {
                fieldsByEntityType[targetType] = [];
              }

              // Add the field without the relationship prefix to the target entity type
              fieldsByEntityType[targetType].push(fieldName);
            } else {
              // Cannot determine target type for this relationship field
              console.warn(`Missing field definitions for relationship: ${relationshipName}`);
              // Show a notification to the user
              showNotification(`Could not determine entity type for relationship field "${field}". Field selection will be ignored in the API request.`, 'warning');
              // Don't add the field to any entity type as it would be formatted incorrectly
              return;
            }
          } else {
            // Cannot find relationship info for this field
            console.warn(`No relationship data found for: ${relationshipName}`);
            // Show a notification to the user
            showNotification(`Could not find relationship data for field "${field}". Make sure the relationship is included.`, 'warning');
            // Don't add the field to any entity type as it would be formatted incorrectly
            return;
          }
        } else {
          // Regular field - add to main entity
          fieldsByEntityType[entityType].push(field);
        }
      });

      // Add fields by entity type to the parameters
      Object.entries(fieldsByEntityType).forEach(([type, fields]) => {
        if (fields.length > 0) {
          apiParams.addFields(type, fields);
        }
      });
    }

    // Helper function to find target entity type from relationship name
    function findTargetEntityTypeFromRelationship(relationshipName) {
      // Only proceed if this relationship is included
      if (includes.includes(relationshipName)) {
        // If we have related entity fields data, extract the type information
        if (relatedEntityFields[relationshipName] && relatedEntityFields[relationshipName].length > 0) {
          // First try to find the base relationship field as it should have the target entity type
          const baseRelationshipField = relatedEntityFields[relationshipName].find(
            field => field.isBaseRelationship === true
          );

          if (baseRelationshipField && baseRelationshipField.targetEntityType) {
            return baseRelationshipField.targetEntityType;
          }

          // If not found in base relationship, try any field from this relationship
          for (const field of relatedEntityFields[relationshipName]) {
            if (field.targetEntityType) {
              return field.targetEntityType;
            }
          }
        }

        // Only hardcode uid as it's a core field with a consistent type
        if (relationshipName === 'uid') {
          return 'user--user';
        }
      }

      // Could not determine target type
      return null;
    }

    // Includes
    if (includes.length > 0) {
      apiParams.addInclude(includes);
    }

    // Add filter groups first (must be created before adding filters to them)
    if (filters.length > 0) {
      // Create a mapping of our group IDs to the names used in the library
      const groupMap = {};

      // First pass: create all groups
      filters.filter(filter => filter.group).forEach(group => {
        // Use the group name as the identifier or fallback to the groupId if no name was provided
        const groupId = group.groupId;
        const groupName = group.groupName || `Group ${groupId}`;

        // Store in our mapping for later use
        groupMap[groupId] = groupName;

        // Add the group to the parameters
        apiParams.addGroup(groupName, group.conjunction);
      });

      // Second pass: add all filters
      filters.forEach(filter => {
        if (filter.group) {
          // Process group members
          if (filter.members && filter.members.length > 0) {
            const groupName = groupMap[filter.groupId];

            filter.members.forEach(memberFilter => {
              // Convert our operator 'value' to '=' for the library
              const operator = memberFilter.operator === 'value' ? '=' : memberFilter.operator;

              // Handle special cases for operators that expect array values
              if (['IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN'].includes(operator) && typeof memberFilter.value === 'string') {
                if (memberFilter.value.includes(',')) {
                  // Split the comma-separated string into an array
                  const values = memberFilter.value.split(',').map(item => item.trim());

                  // Add the filter with the array value to the specified group
                  apiParams.addFilter(memberFilter.field, values, operator, groupName);
                } else {
                  // If no comma, use as single value
                  apiParams.addFilter(memberFilter.field, memberFilter.value, operator, groupName);
                }
              } else {
                // Standard filters go here
                apiParams.addFilter(memberFilter.field, memberFilter.value, operator, groupName);
              }
            });
          }
        } else {
          // Handle standalone filters (not in groups)
          const operator = filter.operator === 'value' ? '=' : filter.operator;

          // Handle special cases for operators that expect array values
          if (['IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN'].includes(operator) && typeof filter.value === 'string') {
            if (filter.value.includes(',')) {
              // Split the comma-separated string into an array
              const values = filter.value.split(',').map(item => item.trim());

              // Add the filter with the array value
              apiParams.addFilter(filter.field, values, operator);
            } else {
              // If no comma, use as single value
              apiParams.addFilter(filter.field, filter.value, operator);
            }
          } else {
            // Standard filters go here
            apiParams.addFilter(filter.field, filter.value, operator);
          }
        }
      });
    }

    // Sorts
    if (sorts.length > 0) {
      sorts.forEach(sort => {
        apiParams.addSort(sort.field, sort.direction.toUpperCase());
      });
    }

    // Pagination
    if (limit !== 10) {
      apiParams.addPageLimit(limit);
    }

    if (page > 0) {
      apiParams.addPageOffset(page * limit);
    }

    // Build the URL with the query parameters
    const queryString = apiParams.getQueryString();
    const url = `${baseUrl}${selectedEntity}/${selectedBundle}${queryString ? '?' + queryString : ''}`;

    setRequestUrl(url);
  };

  const executeRequest = async () => {
    if (!requestUrl) return;

    setLoading(true);
    setError(null);
    setResponse(null);
    setResponseHeaders(null);
    setResponseSize(0);
    setResponseTime(0);

    try {
      const startTime = performance.now();

      // Use default headers - browser will automatically include cookies for authentication
      const headers = {
        'Accept': 'application/vnd.api+json'
      };

      const response = await axios.get(requestUrl, { headers });

      const endTime = performance.now();
      setResponseTime(endTime - startTime);

      if (response.data) {
        setResponse(response.data);

        try {
          // Calculate response size
          const jsonSize = JSON.stringify(response.data).length;
          setResponseSize(jsonSize);
        } catch (error) {
          console.error('Error calculating response size:', error);
          setResponseSize(0);
        }

        // Add to request history
        addToHistory(requestUrl, response.data);
      } else {
        setResponse({});
      }

      if (response.headers) {
        setResponseHeaders(response.headers);
      }

    } catch (err) {
      console.error('Error executing request:', err);
      setError(`Error executing request: ${err.message}`);

      if (err.response) {
        if (err.response.data) {
          setResponse(err.response.data);
        }

        if (err.response.headers) {
          setResponseHeaders(err.response.headers);
        }
      } else {
        // If no response object at all, provide an empty object
        setResponse({
          error: {
            message: err.message,
            name: err.name
          }
        });
      }
    } finally {
      setLoading(false);
    }
  };

  const addToHistory = (url, response) => {
    let responsePreview = 'No response';

    try {
      if (response) {
        responsePreview = JSON.stringify(response).slice(0, 100) + '...';
      }
    } catch (error) {
      console.error('Error stringifying response for history:', error);
      responsePreview = 'Response cannot be displayed';
    }

    const newEntry = {
      id: Date.now(),
      url,
      timestamp: new Date().toISOString(),
      responsePreview
    };

    const updatedHistory = [newEntry, ...requestHistory.slice(0, 9)];
    setRequestHistory(updatedHistory);

    // Persist to localStorage
    storage.set(STORAGE_KEYS.REQUEST_HISTORY, updatedHistory);
  };

  const loadFromHistory = (historyItem) => {
    setRequestUrl(historyItem.url);
    // Parse URL and set all the parameters
    parseUrlToParameters(historyItem.url);
  };

  const parseUrlToParameters = (url) => {
    try {
      // Extract the path and query parameters
      const urlObj = new URL(url);
      const path = urlObj.pathname.split('/');
      const queryString = urlObj.search.startsWith('?') ? urlObj.search.substring(1) : urlObj.search;

      // Extract entity type and bundle from the path
      if (path.length >= 4) {
        const entityPath = path[path.length - 2];
        const bundlePath = path[path.length - 1];

        // Set entity and bundle
        setSelectedEntity(entityPath);
        setSelectedBundle(bundlePath);

        // Use drupal-jsonapi-params to parse the query string
        const apiParams = new DrupalJsonApiParams();
        apiParams.initializeWithQueryString(queryString);
        const params = apiParams.getQueryObject();

        // Process fields (sparse fieldsets)
        if (params.fields) {
          // Find fields for the current entity type
          const entityType = `${entityPath}--${bundlePath}`;
          if (params.fields[entityType]) {
            const fields = params.fields[entityType].split(',');
            setSelectedFields(fields);
          } else {
            setSelectedFields([]);
          }
        } else {
          setSelectedFields([]);
        }

        // Process includes
        if (params.include && params.include.length > 0) {
          const includeFields = Array.isArray(params.include) ? params.include : params.include.split(',');
          setIncludes(includeFields);
        } else {
          setIncludes([]);
        }

        // Process filters - complex part
        if (params.filter) {
          const newFilters = [];
          const groupsMap = {}; // Maps group IDs to filter objects
          const conditionsMap = {}; // Maps condition ID to their memberships

          // Process each filter entry
          Object.entries(params.filter).forEach(([key, value]) => {
            // Case 1: This is a group definition
            if (value.group && value.group.conjunction) {
              const groupId = Date.now() + '_' + key; // Create a unique internal ID
              const groupName = `Group ${newFilters.filter(f => f.group).length + 1}`;

              // Create the group object
              const group = {
                group: true,
                groupId: groupId,
                groupName: groupName,
                conjunction: value.group.conjunction,
                members: [],
                externalId: key // Store original group ID for reference
              };

              groupsMap[key] = group;
              newFilters.push(group);
            }
            // Case 2: This is a condition with a field, operator, and value
            else if (value.condition && value.condition.path) {
              const condition = value.condition;

              // Create a filter object
              const filter = {
                field: condition.path,
                operator: condition.operator || 'value', // Default to value if no operator
                value: condition.value !== undefined ? condition.value : ''
              };

              // Check if this condition belongs to a group
              if (condition.memberOf && groupsMap[condition.memberOf]) {
                // Add to group members
                groupsMap[condition.memberOf].members.push(filter);
              } else {
                // Add as standalone filter
                newFilters.push(filter);
              }
            }
            // Case 3: Simple filter with operator shorthand (filter[title][CONTAINS]=foo)
            else if (typeof value === 'object' && !Array.isArray(value) && value.operator) {
              const filter = {
                field: key,
                operator: value.operator,
                value: value.value || ''
              };
              newFilters.push(filter);
            }
            // Case 4: Simple filter (filter[status]=1)
            else if (typeof value !== 'object') {
              const filter = {
                field: key,
                operator: 'value', // Default operator for simple filters
                value: String(value)
              };
              newFilters.push(filter);
            }
          });

          setFilters(newFilters);
        } else {
          setFilters([]);
        }

        // Process sorts
        if (params.sort && params.sort.length > 0) {
          const sortFields = Array.isArray(params.sort) ? params.sort : params.sort.split(',');
          const newSorts = sortFields.map(field => {
            if (field.startsWith('-')) {
              return { field: field.substring(1), direction: 'desc' };
            } else {
              return { field, direction: 'asc' };
            }
          });

          setSorts(newSorts);
        } else {
          setSorts([]);
        }

        // Process pagination
        if (params.page) {
          if (params.page.limit) {
            setLimit(parseInt(params.page.limit, 10));
          } else {
            setLimit(10);
          }

          if (params.page.offset && params.page.limit) {
            setPage(Math.floor(parseInt(params.page.offset, 10) / parseInt(params.page.limit, 10)));
          } else {
            setPage(0);
          }
        } else {
          setLimit(10);
          setPage(0);
        }
      }
    } catch (err) {
      console.error('Error parsing URL:', err);
      setError('Error parsing URL parameters');
    }
  };

  // Store notification timer in a ref to allow clearing it
  const notificationTimerRef = React.useRef(null);

  const showNotification = (message, type = 'info') => {
    // Clear any existing timer first
    if (notificationTimerRef.current) {
      clearTimeout(notificationTimerRef.current);
      notificationTimerRef.current = null;
    }

    console.log(`Showing notification: ${message} (${type})`);

    // First close any existing notification to trigger a fresh animation
    setNotification(prev => ({
      ...prev,
      open: false
    }));

    // Then set the new notification with a slight delay
    setTimeout(() => {
      setNotification({
        open: true,
        message,
        type
      });

      // Auto-dismiss after 7 seconds
      notificationTimerRef.current = setTimeout(() => {
        handleCloseNotification();
      }, 7000);
    }, 50);
  };

  const handleCloseNotification = () => {
    console.log('Closing notification');

    // Clear any existing timer
    if (notificationTimerRef.current) {
      clearTimeout(notificationTimerRef.current);
      notificationTimerRef.current = null;
    }

    // Hide the notification - directly set state to ensure it updates
    setNotification({
      open: false,
      message: '',
      type: 'info'
    });
  };

  // Function to clear all saved settings
  const clearSavedSettings = () => {
    // Don't clear request history or auth token

    // Clear all selections
    setSelectedEntity('');
    setSelectedBundle('');
    setSelectedFields([]);
    setIncludes([]);
    setFilters([]);
    setSorts([]);
    setPage(0);
    setLimit(10);
    setAutoExecute(false);

    // Clear localStorage for these keys
    Object.keys(STORAGE_KEYS).forEach(keyName => {
      const key = STORAGE_KEYS[keyName];
      // Keep history and auth token
      if (key !== STORAGE_KEYS.REQUEST_HISTORY && key !== STORAGE_KEYS.AUTH_TOKEN) {
        localStorage.removeItem(key);
      }
    });

    showNotification('All saved query settings have been cleared', 'success');
  };

  const handleTabChange = (event, newValue) => {
    setActiveTab(newValue);
  };

  const generateCurlCommand = () => {
    if (!requestUrl) return '';

    let command = `curl -X GET "${requestUrl}"`;

    // Note: we're not including authentication headers as the examples show this separately
    command += ' -H "Accept: application/vnd.api+json"';

    return command;
  };


  const copyToClipboard = (text) => {
    navigator.clipboard.writeText(text).then(() => {
      showNotification('Copied to clipboard!', 'success');
    }).catch(err => {
      showNotification('Failed to copy: ' + err, 'error');
    });
  };

  // If we're rendering the main explorer tab, use the resizable panel layout
  const renderExplorerLayout = () => {
    return (
      <div className="explorer-panels-layout">
        <PanelGroup
          direction="horizontal"
          onLayout={handlePanelResize}
          autoSaveId="jsonapi-explorer-panel-layout">
          {/* Left Sidebar Panel */}
          <Panel defaultSize={panelSizes[0]} minSize={15} className="explorer-panel explorer-panel-sidebar">
            <div className="explorer-panel-header">
              Explorer
            </div>
            <div className="explorer-panel-content" style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
              {loading && !entities.length ? (
                <div className="spinner-container">
                  <div className="spinner"></div>
                </div>
              ) : error && !entities.length ? (
                <div className="messages messages--error">{error}</div>
              ) : (
                <>
                  <div className="entity-panel">
                    <EntityTypeSelector
                      entities={entities}
                      selectedEntity={selectedEntity}
                      selectedBundle={selectedBundle}
                      onEntityChange={setSelectedEntity}
                      onBundleChange={setSelectedBundle}
                    />
                  </div>

                  <div className="panel-section">
                    <PageLimitSelector
                      page={page}
                      limit={limit}
                      onPageChange={setPage}
                      onLimitChange={setLimit}
                    />
                  </div>

                  <RequestHistorySection
                    history={requestHistory}
                    onLoadHistory={loadFromHistory}
                    baseUrl={baseUrl}
                  />
                </>
              )}
            </div>
          </Panel>

          {/* Resize Handle */}
          <PanelResizeHandle className="panel-resizer" />

          {/* Middle Panel - Query Builder */}
          <Panel defaultSize={panelSizes[1]} minSize={25} className="explorer-panel explorer-panel-builder">
            <div className="explorer-panel-header">
              <span>Query Builder</span>
              <div style={{ display: 'flex', alignItems: 'center' }}>
                <div style={{ display: 'flex', alignItems: 'center' }}>
                  <label 
                    style={{ 
                      marginRight: '5px', 
                      display: 'flex', 
                      alignItems: 'center', 
                      cursor: 'pointer',
                      fontSize: '0.9rem'
                    }}
                  >
                    <input
                      type="checkbox"
                      checked={autoExecute}
                      onChange={(e) => setAutoExecute(e.target.checked)}
                      style={{ marginRight: '5px' }}
                    />
                    Auto Execute
                  </label>
                  <div
                    className="field-tooltip-wrapper js-tooltip"
                    data-tooltip="When enabled, automatically executes the query and updates the response each time a change is made to any parameter (includes, fields, filters, etc.)."
                    style={{ marginRight: '10px', cursor: 'help' }}
                  >
                    <span style={{
                      display: 'inline-flex',
                      alignItems: 'center',
                      justifyContent: 'center',
                      width: '16px',
                      height: '16px',
                      borderRadius: '50%',
                      background: 'var(--gin-color-primary-light)',
                      color: 'var(--gin-color-primary)',
                      fontSize: '12px',
                      fontWeight: 'bold'
                    }}>?</span>
                  </div>
                </div>
                <button
                  className="explorer-toolbar-button"
                  onClick={executeRequest}
                  disabled={!requestUrl || loading}
                  data-testid="execute-button">
                  {loading ? <div className="spinner"></div> : "Execute Query"}
                </button>
              </div>
            </div>

            {/* URL Preview always displayed before tabs */}
            <div className="url-display-container">
              <UrlDisplay
                url={requestUrl}
                onCopy={() => copyToClipboard(requestUrl)}
                showNotification={showNotification}
              />
            </div>

            <div className="query-panel-tabs">
              <button
                className={`query-panel-tab ${builderPanelActiveTab === 'fields' ? 'active' : ''}`}
                onClick={() => setBuilderPanelActiveTab('fields')}>
                Fields
              </button>
              <button
                className={`query-panel-tab ${builderPanelActiveTab === 'filters' ? 'active' : ''}`}
                onClick={() => setBuilderPanelActiveTab('filters')}>
                Filters
              </button>
              <button
                className={`query-panel-tab ${builderPanelActiveTab === 'includes' ? 'active' : ''}`}
                onClick={() => setBuilderPanelActiveTab('includes')}>
                Includes
              </button>
              <button
                className={`query-panel-tab ${builderPanelActiveTab === 'sort' ? 'active' : ''}`}
                onClick={() => setBuilderPanelActiveTab('sort')}>
                Sort
              </button>
              <button
                className={`query-panel-tab ${builderPanelActiveTab === 'code' ? 'active' : ''}`}
                onClick={() => setBuilderPanelActiveTab('code')}>
                &lt;/&gt;
              </button>
            </div>

            <div className="explorer-panel-content">
              {/* Different content based on selected tab */}
              {builderPanelActiveTab === 'fields' && (
                <div className="tab-content-section">
                  <FieldsSelector
                    key={`fields-${selectedEntity}-${selectedBundle}-${includes.join('-')}`}
                    selectedEntity={selectedEntity}
                    selectedBundle={selectedBundle}
                    selectedFields={selectedFields}
                    onFieldsChange={setSelectedFields}
                    relatedEntityFields={relatedEntityFields}
                    loadingRelatedFields={loadingRelatedFields}
                    baseUrl={baseUrl}
                  />
                </div>
              )}

              {builderPanelActiveTab === 'filters' && (
                <div className="tab-content-section">
                  <FilterSelector
                    selectedEntity={selectedEntity}
                    selectedBundle={selectedBundle}
                    filters={filters}
                    onFiltersChange={setFilters}
                    relatedEntityFields={relatedEntityFields}
                    baseUrl={baseUrl}
                  />
                </div>
              )}

              {builderPanelActiveTab === 'includes' && (
                <div className="tab-content-section">
                  <IncludeSelector
                    selectedEntity={selectedEntity}
                    selectedBundle={selectedBundle}
                    includes={includes}
                    onIncludeChange={(newIncludes) => {
                      // First set flag to indicate user action
                      userActionRef.current = true;

                      // Then update includes (this will trigger our useEffect)
                      setIncludes(newIncludes);
                    }}
                    baseUrl={baseUrl}
                  />
                </div>
              )}

              {builderPanelActiveTab === 'sort' && (
                <div className="tab-content-section">
                  <SortSelector
                    selectedEntity={selectedEntity}
                    selectedBundle={selectedBundle}
                    sorts={sorts}
                    onSortsChange={setSorts}
                    relatedEntityFields={relatedEntityFields}
                    baseUrl={baseUrl}
                  />
                </div>
              )}

              {builderPanelActiveTab === 'code' && (
                <div className="tab-content-section">
                  <CodeExamplesTab
                    requestUrl={requestUrl}
                    onCopy={(code) => copyToClipboard(code)}
                    showNotification={showNotification}
                  />
                </div>
              )}
            </div>
          </Panel>

          {/* Resize Handle */}
          <PanelResizeHandle className="panel-resizer" />

          {/* Right Panel - Response */}
          <Panel defaultSize={panelSizes[2]} minSize={20} className="explorer-panel explorer-panel-response">
            <div className="explorer-panel-header">
              <span>Response</span>
              {responseTime > 0 && responseSize > 0 && (
                <div className="explorer-response-stats">
                  <span>
                    {responseTime < 1000
                      ? `${Math.round(responseTime)}ms`
                      : `${(responseTime / 1000).toFixed(2)}s`}
                  </span>
                  <span className="ml-2">{(responseSize / 1024).toFixed(2)} KB</span>
                </div>
              )}
            </div>

            <div className="explorer-panel-content">
              {/* Error message display */}
              {error ? <div className="messages messages--error">{error}</div> : null}

              {/* Response display */}
              <ResponseDisplay
                response={response}
                responseHeaders={responseHeaders}
                onCopyJson={() => {
                  if (response) {
                    try {
                      copyToClipboard(JSON.stringify(response, null, 2));
                    } catch (err) {
                      console.error('Error copying response:', err);
                      try {
                        copyToClipboard(String(response));
                      } catch (finalErr) {
                        console.error('Final error copying response:', finalErr);
                      }
                    }
                  }
                }}
                onCopyPath={(path) => {
                  if (path) {
                    copyToClipboard(path);
                  }
                }}
              />
            </div>
          </Panel>
        </PanelGroup>
      </div>
    );
  };

  // Helper function to render a code example accordion
  const renderCodeAccordion = (title, key, code, language = 'bash') => {
    return (
      <div style={{ 
        marginBottom: '12px',
        border: '1px solid var(--explorer-border)',
        borderRadius: '4px',
        overflow: 'hidden'
      }}>
        <div 
          style={{
            padding: '12px 16px',
            backgroundColor: 'var(--explorer-background-secondary)',
            borderBottom: expandedExample === key ? '1px solid var(--explorer-border)' : 'none',
            cursor: 'pointer',
            display: 'flex',
            justifyContent: 'space-between',
            alignItems: 'center',
            fontWeight: '500'
          }}
          onClick={() => setExpandedExample(expandedExample === key ? '' : key)}
        >
          <span>{title}</span>
          {expandedExample === key ? (
            <button
              className="button button--small"
              onClick={(e) => {
                e.stopPropagation();
                copyToClipboard(code);
              }}
              title="Copy example code"
            >
              Copy
            </button>
          ) : (
            <span style={{ fontSize: '0.9em', opacity: 0.6 }}>Click to expand</span>
          )}
        </div>
        
        {expandedExample === key && (
          <div style={{ 
            display: 'block',
            padding: '0',
            backgroundColor: 'var(--explorer-background-tertiary)'
          }}>
            <SyntaxHighlighter
              language={language}
              style={materialLight}
              wrapLines={true}
              customStyle={{
                fontSize: '0.8rem',
                margin: 0,
                padding: '16px',
                backgroundColor: 'var(--explorer-background-tertiary)'
              }}
            >
              {code}
            </SyntaxHighlighter>
          </div>
        )}
      </div>
    );
  };

  // Tab panels content
  const renderTabPanel = (index) => {
    switch (index) {
      case 0: // Build Query Tab
        return renderExplorerLayout();

      case 1: // Authentication Tab
        return (
          <div className="layout-row" data-testid="auth-tab">
            <div className="layout-column layout-column--full">
              <div className="panel">
                <div className="gin-layer-wrapper">
                  <h3 className="heading">Authentication</h3>

                  {/* Authentication info */}
                  <div className="form-item">
                    <div className="form-item__description">
                      The JSON:API Explorer automatically uses your current Drupal session for authentication.
                      No additional authentication is required when using this tool.

                      The examples below show how to authenticate when accessing the API from external applications.
                    </div>
                  </div>

                  {/* Using the updated AuthSection component */}
                  <AuthSection baseUrl={baseUrl} />
                </div>
              </div>
            </div>
          </div>
        );

      case 2: // Examples Tab
        return (
          <div className="layout-row" data-testid="code-examples-tab">
            <div className="layout-column layout-column--full">
              <div className="panel">
                <div className="gin-layer-wrapper">
                  <h3 className="heading">Advanced JSON:API Examples</h3>

                  <div style={{ display: 'flex', flexDirection: 'column', gap: '12px' }}>
                    {/* Render all examples using our new helper function */}
                    {renderCodeAccordion('Basic Filtering', 'filtering', 
                      `# Basic filter examples
curl -X GET "${baseUrl}node/article?filter[status]=1&filter[title][CONTAINS]=Example" \\
  -H "Accept: application/vnd.api+json"

# Filter with operators
curl -X GET "${baseUrl}node/article?filter[created][>]=2023-01-01" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Advanced Filtering with Groups', 'advanced_filtering', 
                      `# Filter with AND/OR groups (complex conditions)
# Find articles that are either (published AND contain "news") OR (have a specific tag)

curl -X GET "${baseUrl}node/article?\\
filter[group1][conjunction]=AND\\
&filter[condition1][condition][memberOf]=group1\\
&filter[condition1][condition][field]=status\\
&filter[condition1][condition][value]=1\\
&filter[condition2][condition][memberOf]=group1\\
&filter[condition2][condition][field]=title\\
&filter[condition2][condition][operator]=CONTAINS\\
&filter[condition2][condition][value]=news\\
&filter[group2][conjunction]=OR\\
&filter[group2][memberOf]=\\
&filter[condition3][condition][memberOf]=group2\\
&filter[condition3][condition][field]=field_tags.name\\
&filter[condition3][condition][value]=featured" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Including Related Resources', 'includes', 
                      `# Include related entities
curl -X GET "${baseUrl}node/article?include=uid,field_tags" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Sparse Fieldsets', 'fields', 
                      `# Request only specific fields
curl -X GET "${baseUrl}node/article?fields[node--article]=title,created,status&fields[user--user]=name" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Sorting Results', 'sorting', 
                      `# Sort by multiple fields
curl -X GET "${baseUrl}node/article?sort=-created,title" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Pagination', 'pagination', 
                      `# Paginate results
curl -X GET "${baseUrl}node/article?page[offset]=10&page[limit]=5" \\
  -H "Accept: application/vnd.api+json"`
                    )}
                    
                    {renderCodeAccordion('Creating Relationships', 'relationships', 
                      `# Create content with relationships
curl -X POST "${baseUrl}node/article" \\
  -H "Authorization: Bearer YOUR_ACCESS_TOKEN" \\
  -H "Content-Type: application/vnd.api+json" \\
  -H "Accept: application/vnd.api+json" \\
  -d '{
    "data": {
      "type": "node--article",
      "attributes": {
        "title": "Article with Tags",
        "body": {
          "value": "This article has tags attached.",
          "format": "full_html"
        }
      },
      "relationships": {
        "field_tags": {
          "data": [
            {
              "type": "taxonomy_term--tags",
              "id": "TAG_UUID_HERE"
            }
          ]
        }
      }
    }
  }'`
                    )}
                    
                    {renderCodeAccordion('File Upload Example', 'file-upload', 
                      `# Create a file entity (2-step process)
# Step 1: Upload the file
curl -X POST "${baseUrl}file/file" \\
  -H "Authorization: Bearer YOUR_ACCESS_TOKEN" \\
  -H "Content-Type: application/octet-stream" \\
  -H "Content-Disposition: file; filename=\\"example.jpg\\"" \\
  -H "Accept: application/vnd.api+json" \\
  --data-binary @/path/to/example.jpg

# Step 2: Create a media entity referencing the file
curl -X POST "${baseUrl}media/image" \\
  -H "Authorization: Bearer YOUR_ACCESS_TOKEN" \\
  -H "Content-Type: application/vnd.api+json" \\
  -H "Accept: application/vnd.api+json" \\
  -d '{
    "data": {
      "type": "media--image",
      "attributes": {
        "name": "My Image"
      },
      "relationships": {
        "field_media_image": {
          "data": {
            "type": "file--file",
            "id": "FILE_UUID_FROM_STEP_1"
          }
        }
      }
    }
  }'`
                    )}
                  </div>
                </div>
              </div>
            </div>
          </div>
        );

      default:
        return null;
    }
  };

  return (
    <div className="jsonapi-explorer-container">
      <div className="layout-container">
        {/* Tab navigation - Gin style */}
        <nav className="tabs-wrapper is-horizontal is-collapsible" role="navigation" aria-labelledby="primary-tabs-title">
          <h2 className="visually-hidden" id="primary-tabs-title">Primary tabs</h2>
          <ul className="tabs primary">
            <li className={`tabs__tab ${activeTab === 0 ? 'is-active' : ''}`}>
              <button
                className="tabs__link"
                role="tab"
                aria-selected={activeTab === 0 ? 'true' : 'false'}
                onClick={() => setActiveTab(0)}
                data-tab="builder"
              >
                Query Builder
              </button>
            </li>
            <li className={`tabs__tab ${activeTab === 1 ? 'is-active' : ''}`}>
              <button
                className="tabs__link"
                role="tab"
                aria-selected={activeTab === 1 ? 'true' : 'false'}
                onClick={() => setActiveTab(1)}
                data-tab="auth"
              >
                Authentication
              </button>
            </li>
            <li className={`tabs__tab ${activeTab === 2 ? 'is-active' : ''}`}>
              <button
                className="tabs__link"
                role="tab"
                aria-selected={activeTab === 2 ? 'true' : 'false'}
                onClick={() => setActiveTab(2)}
                data-tab="code-examples"
              >
                Examples
              </button>
            </li>
          </ul>
        </nav>

        {/* Tab content */}
        <div className="tabs__content">
          {renderTabPanel(activeTab)}
        </div>
      </div>

      {/* Notification container - fixed position for consistent placement */}
      <div className="notification-container" style={{
        position: 'fixed',
        bottom: '40px',
        left: '0',
        right: '0',
        zIndex: 10000,
        display: 'flex',
        justifyContent: 'center',
        pointerEvents: 'none', // Let clicks pass through container
        width: '100%'
      }}>
        <div
          className={`messages messages--${notification.type === 'status' ? 'info' : notification.type}`}
          role="alert"
          style={{
            opacity: notification.open ? 1 : 0,
            transform: notification.open ? 'translateY(0)' : 'translateY(20px)',
            transition: 'opacity 0.3s ease-out, transform 0.3s ease-out',
            visibility: notification.open ? 'visible' : 'hidden',
            pointerEvents: notification.open ? 'auto' : 'none', // Only capture clicks when open
            borderWidth: '0 0 0 4px', // Left border only
            maxWidth: '500px',
            width: 'auto',
            margin: '0 auto',
            boxShadow: '0 4px 12px rgba(0, 0, 0, 0.15)'
          }}
        >
          <div className="messages__content">
            <span>{notification.message}</span>
            <button
              className="messages__close"
              onClick={() => handleCloseNotification()}
              aria-label="Close"
            >×</button>
          </div>
        </div>
      </div>
    </div>
  );
}

export default App;
