<?php

namespace Drupal\Tests\jsonapi_query_builder\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Tests the functionality of the JSON:API Query Builder module.
 *
 * @group jsonapi_query_builder
 */
class JsonApiQueryBuilderTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'jsonapi',
    'jsonapi_schema',
    'jsonapi_query_builder',
    'node',
  ];

  /**
   * A test user with permission to access the JSON:API Explorer.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $privilegedUser;

  /**
   * A test user without permission to access the JSON:API Explorer.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $unprivilegedUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create test content type and content.
    $this->createContentType(['type' => 'article', 'name' => 'Article']);
    $this->createNode([
      'title' => 'Test Article 1',
      'type' => 'article',
      'status' => 1,
    ]);
    $this->createNode([
      'title' => 'Test Article 2',
      'type' => 'article',
      'status' => 1,
    ]);

    // Create users.
    $this->privilegedUser = $this->drupalCreateUser(['access jsonapi query builder']);
    $this->unprivilegedUser = $this->drupalCreateUser();

    // Ensure JSON:API is enabled for all entities.
    $this->config('jsonapi.settings')
      ->set('read_only', FALSE)
      ->save();
  }

  /**
   * Tests that the module's routes are accessible to users with permission.
   */
  public function testRouteAccess() {
    // Test access for privileged user.
    $this->drupalLogin($this->privilegedUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('JSON:API Explorer');

    // Test for the React app container.
    $this->assertSession()->elementExists('css', '#jsonapi-explorer-root');

    // Test access denied for unprivileged user.
    $this->drupalLogin($this->unprivilegedUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests that the module correctly loads its libraries.
   */
  public function testLibraryLoading() {
    $this->drupalLogin($this->privilegedUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Test for CSS and JS from the module.
    $this->assertSession()->responseContains('jsonapi_query_builder/explorer');

    // Check that drupalSettings are loaded.
    $this->assertSession()->responseContains('drupalSettings.jsonapi_explorer');
  }

  /**
   * Tests that JSON:API is correctly configured and accessible.
   */
  public function testJsonApiAccess() {
    // Allow anonymous access to the JSON:API.
    Role::load(RoleInterface::ANONYMOUS_ID)
      ->grantPermission('access jsonapi resource list')
      ->grantPermission('access content')
      ->save();

    // Test JSON:API index endpoint.
    $this->drupalGet('/jsonapi');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('links');

    // Test JSON:API schema support.
    $this->drupalGet('/jsonapi/node/article/resource/schema');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('properties');
  }

  /**
   * Tests that JSON:API Query Builder integrates with the Drupal toolbar.
   */
  public function testToolbarIntegration() {
    // Create a user with permission to use the toolbar.
    $toolbar_user = $this->drupalCreateUser([
      'access jsonapi query builder',
      'access toolbar',
    ]);

    $this->drupalLogin($toolbar_user);
    $this->drupalGet('<front>');

    // Check for the toolbar link to the JSON:API Explorer.
    $this->assertSession()->responseContains('jsonapi-explorer');
  }

  /**
   * Tests the user permissions work correctly.
   */
  public function testPermissions() {
    // Check that the permission is defined.
    $permissions = $this->container->get('user.permissions')
      ->getPermissions();
    $this->assertArrayHasKey('access jsonapi query builder', $permissions);
  }

}
