<?php

namespace Drupal\Tests\jsonapi_query_builder\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Core\Url;

/**
 * Tests the JavaScript functionality of the JSON:API Query Builder.
 *
 * @group jsonapi_query_builder
 */
class JsonApiQueryBuilderJsTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'jsonapi',
    'jsonapi_schema',
    'jsonapi_query_builder',
    'node',
    'field_ui',
  ];

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create test content type and content.
    $this->createContentType(['type' => 'article', 'name' => 'Article']);

    // Add a field to the article content type.
    $this->drupalCreateContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    // Create test content.
    $this->createNode([
      'title' => 'Test Article 1',
      'type' => 'article',
      'body' => [
        'value' => 'Test body content',
        'format' => 'plain_text',
      ],
      'status' => 1,
    ]);

    $this->createNode([
      'title' => 'Test Article 2',
      'type' => 'article',
      'body' => [
        'value' => 'Another test article',
        'format' => 'plain_text',
      ],
      'status' => 1,
    ]);

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'access jsonapi query builder',
      'access content',
      'administer nodes',
      'administer site configuration',
      'administer users',
      'access content overview',
      'access administration pages',
    ]);

    // Ensure JSON:API is enabled for all entities.
    $this->config('jsonapi.settings')
      ->set('read_only', FALSE)
      ->save();
  }

  /**
   * Tests that the React application loads and functions correctly.
   */
  public function testReactAppLoads() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '#jsonapi-query-builder-root .entity-type-selector');

    // Verify UI elements are present.
    $this->assertSession()->elementExists('css', '.entity-type-selector');
    $this->assertSession()->elementExists('css', '.query-panel-tab[data-tab="fields"]');
    $this->assertSession()->elementExists('css', '.query-panel-tab[data-tab="filters"]');
    $this->assertSession()->elementExists('css', '.query-panel-tab[data-tab="includes"]');
    $this->assertSession()->elementExists('css', '.query-panel-tab[data-tab="sort"]');

    // Verify URL display is present.
    $this->assertSession()->elementExists('css', '.url-display');

    // Verify execute button is present.
    $this->assertSession()->elementExists('css', '.execute-button');
  }

  /**
   * Tests that entity types and bundles can be selected.
   */
  public function testEntitySelection() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '.entity-type-selector');

    // Select node entity type.
    $page = $this->getSession()->getPage();
    $entityTypeSelect = $page->find('css', '.entity-type-selector select.entity-type');
    $this->assertNotNull($entityTypeSelect, 'Entity type select not found');

    $entityTypeSelect->selectOption('node');

    // Wait for bundle select to appear.
    $bundleSelect = $this->assertSession()->waitForElement('css', '.entity-type-selector select.entity-bundle');
    $this->assertNotNull($bundleSelect, 'Bundle select not found after entity type selection');

    // Select article bundle.
    $bundleSelect->selectOption('article');

    // Wait for URL to update.
    $this->assertSession()->waitForText('/jsonapi/node/article');

    // Verify URL contains correct path.
    $this->assertSession()->pageTextContains('/jsonapi/node/article');
  }

  /**
   * Tests that fields can be selected and deselected.
   */
  public function testFieldsSelection() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '.entity-type-selector');

    // Select node--article.
    $page = $this->getSession()->getPage();
    $page->find('css', '.entity-type-selector select.entity-type')->selectOption('node');
    $this->assertSession()->waitForElement('css', '.entity-type-selector select.entity-bundle');
    $page->find('css', '.entity-type-selector select.entity-bundle')->selectOption('article');

    // Wait for fields to load.
    $this->assertSession()->waitForElement('css', '.fields-list');

    // Find and click the title field checkbox.
    $titleCheckbox = $page->find('css', 'input[name="field-title"]');
    $this->assertNotNull($titleCheckbox, 'Title field checkbox not found');
    $titleCheckbox->click();

    // Wait for URL to update.
    $this->assertSession()->waitForText('fields[node--article]=title');

    // Verify URL contains fields parameter.
    $this->assertSession()->pageTextContains('fields[node--article]=title');

    // Click the body field checkbox.
    $bodyCheckbox = $page->find('css', 'input[name="field-body"]');
    $this->assertNotNull($bodyCheckbox, 'Body field checkbox not found');
    $bodyCheckbox->click();

    // Wait for URL to update.
    $this->assertSession()->waitForText('fields[node--article]=title,body');

    // Verify URL contains both fields.
    $this->assertSession()->pageTextContains('fields[node--article]=title,body');

    // Deselect the title field.
    $titleCheckbox->click();

    // Wait for URL to update.
    $this->assertSession()->waitForText('fields[node--article]=body');

    // Verify URL only contains body field.
    $this->assertSession()->pageTextContains('fields[node--article]=body');
  }

  /**
   * Tests that filters can be added and removed.
   */
  public function testFilters() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '.entity-type-selector');

    // Select node--article.
    $page = $this->getSession()->getPage();
    $page->find('css', '.entity-type-selector select.entity-type')->selectOption('node');
    $this->assertSession()->waitForElement('css', '.entity-type-selector select.entity-bundle');
    $page->find('css', '.entity-type-selector select.entity-bundle')->selectOption('article');

    // Wait for fields to load.
    $this->assertSession()->waitForElement('css', '.fields-list');

    // Click the filters tab.
    $page->find('css', '.query-panel-tab[data-tab="filters"]')->click();

    // Wait for filter panel to appear.
    $this->assertSession()->waitForElement('css', '.filter-selector');

    // Click add filter button.
    $page->find('css', '.add-filter-button')->click();

    // Wait for filter form to appear.
    $this->assertSession()->waitForElement('css', '.filter-form');

    // Select title field.
    $page->find('css', '.filter-field-select')->selectOption('title');

    // Select CONTAINS operator.
    $page->find('css', '.filter-operator-select')->selectOption('CONTAINS');

    // Enter filter value.
    $page->find('css', '.filter-value-input')->setValue('Test');

    // Click apply button.
    $page->find('css', '.apply-filter-button')->click();

    // Wait for URL to update.
    $this->assertSession()->waitForText('filter[titleFilter][condition][path]=title');

    // Verify URL contains filter parameter.
    $this->assertSession()->pageTextContains('filter[titleFilter][condition][path]=title');
    $this->assertSession()->pageTextContains('filter[titleFilter][condition][operator]=CONTAINS');
    $this->assertSession()->pageTextContains('filter[titleFilter][condition][value]=Test');

    // Find and click the remove filter button.
    $page->find('css', '.remove-filter-button')->click();

    // Wait for URL to update (filter should be removed).
    $this->assertSession()->waitForElementRemoved('css', '.filter-item');

    // Verify URL no longer contains filter parameter.
    $this->assertSession()->pageTextNotContains('filter[titleFilter]');
  }

  /**
   * Tests that includes can be added and removed.
   */
  public function testIncludes() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '.entity-type-selector');

    // Select node--article.
    $page = $this->getSession()->getPage();
    $page->find('css', '.entity-type-selector select.entity-type')->selectOption('node');
    $this->assertSession()->waitForElement('css', '.entity-type-selector select.entity-bundle');
    $page->find('css', '.entity-type-selector select.entity-bundle')->selectOption('article');

    // Wait for fields to load.
    $this->assertSession()->waitForElement('css', '.fields-list');

    // Click the includes tab.
    $page->find('css', '.query-panel-tab[data-tab="includes"]')->click();

    // Wait for includes panel to appear.
    $this->assertSession()->waitForElement('css', '.include-selector');

    // Click add include button.
    $page->find('css', '.add-include-button')->click();

    // Wait for include form to appear.
    $this->assertSession()->waitForElement('css', '.include-form');

    // Select uid relationship.
    $page->find('css', '.include-field-select')->selectOption('uid');

    // Click apply button.
    $page->find('css', '.apply-include-button')->click();

    // Wait for URL to update.
    $this->assertSession()->waitForText('include=uid');

    // Verify URL contains include parameter.
    $this->assertSession()->pageTextContains('include=uid');

    // Find and click the remove include button.
    $page->find('css', '.remove-include-button')->click();

    // Wait for URL to update (include should be removed).
    $this->assertSession()->waitForElementRemoved('css', '.include-item');

    // Verify URL no longer contains include parameter.
    $this->assertSession()->pageTextNotContains('include=uid');
  }

  /**
   * Tests that the API request can be executed and shows a response.
   */
  public function testExecuteRequest() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('jsonapi_query_builder.app'));

    // Wait for the React app to initialize.
    $this->assertSession()->waitForElement('css', '.entity-type-selector');

    // Select node--article.
    $page = $this->getSession()->getPage();
    $page->find('css', '.entity-type-selector select.entity-type')->selectOption('node');
    $this->assertSession()->waitForElement('css', '.entity-type-selector select.entity-bundle');
    $page->find('css', '.entity-type-selector select.entity-bundle')->selectOption('article');

    // Wait for fields to load.
    $this->assertSession()->waitForElement('css', '.fields-list');

    // Click execute request button.
    $page->find('css', '.execute-button')->click();

    // Wait for response to load.
    $this->assertSession()->waitForElement('css', '.response-display');

    // Verify response contains data.
    $this->assertSession()->elementTextContains('css', '.response-display', 'data');
    $this->assertSession()->elementTextContains('css', '.response-display', 'type');
    $this->assertSession()->elementTextContains('css', '.response-display', 'node--article');

    // Verify response contains article titles.
    $this->assertSession()->elementTextContains('css', '.response-display', 'Test Article 1');
  }

}
